==> thx_role-0.1.7-py3-none-any.whl: thx/role/models.py <==
# -*- coding: utf-8 -*-

import django
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models
if django.VERSION < (3, 1):
    from django.contrib.postgres.fields import JSONField
else:
    from django.db.models import JSONField
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _
from mptt.fields import TreeForeignKey
from mptt.models import MPTTModel

from thx.core.models import UserModel, DateModel, StatusModel, OrderedModel, CleanModel


User = get_user_model()


class Role(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    @classmethod
    def get_roles(cls, user):
        return user.roles.all()

    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    app_label = models.CharField(
        max_length=100,
        verbose_name=_('app label')
    )
    model_name = models.CharField(
        max_length=100,
        verbose_name=_('model name')
    )
    codename = models.CharField(
        max_length=255,
        verbose_name=_('codename'),
    )
    users = models.ManyToManyField(
        User,
        related_name="roles",
        blank=True,
        verbose_name=_('users'),
    )

    class Meta:
        verbose_name = _('role')
        verbose_name_plural = _('roles')
        unique_together = (('app_label', 'model_name', 'codename'),)
        ordering = ('ordering', 'app_label', 'model_name', 'codename')
        permissions = (
            ("list_role", "Can list role"),
            ("detail_role", "Can detail role"),
            ("enable_role", "Can enable role"),
            ("disable_role", "Can disable role"),
        )

    def __str__(self):
        return "{} | {} | {}".format(
            self.app_label,
            self.model_name,
            self.name,
        )

    def clean_fields(self, exclude=None):
        self.set_codename()

    def clean(self):
        self.set_codename()
        self.set_app_label()
        self.set_model_name()
        super().clean()

    def set_codename(self):
        self.codename = slugify(self.name)

    def set_app_label(self):
        self.app_label = self.app_label.lower()

    def set_model_name(self):
        self.model_name = self.model_name.lower()


class UserRole(Role.users.through):
    class Meta:
        proxy = True
        ordering = ('role__app_label', 'role__model_name', 'role__name', 'role__name', 'user__username')

    @property
    def username(self):
        return self.user.username

    @property
    def email(self):
        return self.user.email

    @property
    def status(self):
        return self.role.status

    @property
    def app_label(self):
        return self.role.app_label

    @property
    def model_name(self):
        return self.role.model_name

    @property
    def name(self):
        return self.role.name


class RequiredField(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    role = models.ForeignKey(
        Role,
        on_delete=models.CASCADE,
        related_name='required_fields',
        verbose_name=_('role'),
    )
    from_status = models.CharField(
        max_length=50,
        verbose_name=_('from status'),
        help_text=_('e.g.: 1 -> "1", "create" -> "create"')
    )
    fields = JSONField(
        verbose_name=_('fields'),
        help_text=_('e.g.: ["field1", "field2"]')
    )

    class Meta:
        verbose_name = _('required field')
        verbose_name_plural = _('required fields')
        ordering = ('ordering', 'role__app_label', 'role__model_name', 'role__name', 'from_status')
        unique_together = (('role', 'from_status',),)
        permissions = (
            ("list_requiredfield", "Can list required field"),
            ("detail_requiredfield", "Can detail required field"),
            ("enable_requiredfield", "Can enable required field"),
            ("disable_requiredfield", "Can disable required field"),
        )

    def __str__(self):
        return "{} | {} | {} | {}".format(
            self.role.app_label,
            self.role.model_name,
            self.role.name,
            self.from_status
        )

    def clean(self):
        self.set_from_status()
        super().clean()

    def set_from_status(self):
        self.from_status = str(self.from_status)

    @property
    def app_label(self):
        return self.role.app_label

    @property
    def model_name(self):
        return self.role.model_name


class StatusFlow(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    role = models.ForeignKey(
        Role,
        on_delete=models.CASCADE,
        related_name='status_flows',
        verbose_name=_('role'),
    )
    from_status = models.CharField(
        max_length=50,
        verbose_name=_('from status'),
        help_text=_('e.g.: 1 -> "1", "create" -> "create"'),
    )
    to_status = JSONField(
        verbose_name=_('to status'),
        help_text=_('e.g.: ["status1", "status2"]'),
    )

    class Meta:
        verbose_name = _('status flow')
        verbose_name_plural = _('status flows')
        ordering = ('ordering', 'role__app_label', 'role__model_name', 'role__name', 'from_status')
        unique_together = (('role', 'from_status',),)
        permissions = (
            ("list_statusflow", "Can list status flow"),
            ("detail_statusflow", "Can detail status flow"),
            ("enable_statusflow", "Can enable status flow"),
            ("disable_statusflow", "Can disable status flow"),
        )

    def __str__(self):
        return "{} | {} | {} | {}".format(
            self.role.app_label,
            self.role.model_name,
            self.role.name,
            self.from_status
        )

    def clean(self):
        self.set_from_status()
        super().clean()

    def set_from_status(self):
        self.from_status = str(self.from_status)

    @property
    def app_label(self):
        return self.role.app_label

    @property
    def model_name(self):
        return self.role.model_name


class UsedField(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    role = models.ForeignKey(
        Role,
        on_delete=models.CASCADE,
        related_name='used_fields',
        verbose_name=_('role'),
    )
    from_status = models.CharField(
        max_length=50,
        verbose_name=_('from status'),
        help_text=_('e.g.: 1 -> "1", "create" -> "create"'),
    )
    action = models.CharField(
        max_length=50,
        verbose_name=_('action'),
        help_text=_('e.g.: "create", "enable", "destroy"'),
    )
    fields = JSONField(
        verbose_name=_('fields'),
        help_text=_('e.g.: ["field1", "field2"]')
    )

    class Meta:
        verbose_name = _('used field')
        verbose_name_plural = _('used fields')
        unique_together = (('role', 'from_status', 'action',),)
        ordering = (
            'ordering', 'role__app_label', 'role__model_name',
            'role__name', 'from_status', 'action'
        )
        permissions = (
            ("list_usedfield", "Can list used field"),
            ("detail_usedfield", "Can detail used field"),
            ("enable_usedfield", "Can enable used field"),
            ("disable_usedfield", "Can disable used field"),
        )

    def __str__(self):
        return "{} | {} | {} | {} | {}".format(
            self.role.app_label,
            self.role.model_name,
            self.role.name,
            self.from_status,
            self.action,
        )

    def clean(self):
        self.set_from_status()
        super().clean()

    def set_from_status(self):
        self.from_status = str(self.from_status)

    @property
    def app_label(self):
        return self.role.app_label

    @property
    def model_name(self):
        return self.role.model_name


class Filter(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    role = models.ForeignKey(
        Role,
        on_delete=models.CASCADE,
        related_name='filters',
        verbose_name=_('role'),
    )
    action = models.CharField(
        max_length=50,
        verbose_name=_('action'),
        help_text=_('e.g.: "create", "enable", "destroy"'),
    )
    codename = models.CharField(
        max_length=255,
        verbose_name=_('codename'),
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_('description'),
    )
    code = models.TextField(
        default="Q()",
        verbose_name=_('code'),
    )

    class Meta:
        verbose_name = _('filter')
        verbose_name_plural = _('filters')
        unique_together = (('role', 'codename'),)
        ordering = ('ordering', 'role__app_label', 'role__model_name', 'codename')
        permissions = (
            ("list_filter", "Can list filter"),
            ("detail_filter", "Can detail filter"),
            ("enable_filter", "Can enable filter"),
            ("disable_filter", "Can disable filter"),
        )

    def __str__(self):
        return "{} | {} | {}".format(
            self.role.app_label,
            self.role.model_name,
            self.action,
        )

    def clean_fields(self, exclude=None):
        self.set_codename()
        super().clean_fields(exclude)

    def set_codename(self):
        self.codename = slugify(self.action)

    @property
    def app_label(self):
        return self.role.app_label

    @property
    def model_name(self):
        return self.role.model_name


class Menu(MPTTModel, CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    parent = TreeForeignKey(
        "self",
        on_delete=models.CASCADE,
        related_name='children',
        null=True, blank=True,
        verbose_name=_('parent'),
    )
    title = models.CharField(
        max_length=255,
        verbose_name=_('title'),
    )
    language_code = models.CharField(
        max_length=2,
        verbose_name=_('language code'),
    )
    router = models.CharField(
        max_length=255,
        verbose_name=_('router'),
    )
    roles = models.ManyToManyField(
        Role,
        related_name='menus',
        blank=True,
        verbose_name=_('roles'),
    )
    info = JSONField(
        blank=True, null=True,
        verbose_name=_('info'),
    )

    class Meta:
        verbose_name = _('menu')
        verbose_name_plural = _('menu')
        ordering = ('ordering', 'language_code', 'parent__title', 'title')
        permissions = (
            ("list_menu", "Can list menu"),
            ("detail_menu", "Can detail menu"),
            ("enable_menu", "Can enable menu"),
            ("disable_menu", "Can disable menu"),
        )

    class MPTTMeta:
        parent_attr = 'parent'

    def __str__(self):
        return "{} | {}".format(
            self.parent,
            self.title,
        )

    def clean(self):
        self.set_language_code()
        super().clean()

    @classmethod
    def get_user_menu(cls, user, language_code, codename=None):
        language_code = language_code.lower()
        show_all_menus = getattr(settings, 'SHOW_ALL_MENUS', None)
        if show_all_menus:
            return Menu.objects.filter(language_code=language_code, status=1)
        items = Menu.objects.filter(roles__users=user, language_code=language_code, status=1)
        if codename:
            items = items.filter(roles__codename=codename)
        return items.distinct()

    def set_language_code(self):
        self.language_code = self.language_code.lower()

    def role_list(self):
        return [
            f"{obj.app_label} | {obj.model_name} | {obj.codename}"
            for obj in self.roles.filter(status=1)
        ]

    @property
    def parent_title(self):
        return self.parent.title if self.parent else None

==> thx_role-0.1.8-py3-none-any.whl: thx/role/models.py <==
# -*- coding: utf-8 -*-

import django
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models

if django.VERSION < (3, 1):
    from django.contrib.postgres.fields import JSONField
else:
    from django.db.models import JSONField

from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _
from mptt.fields import TreeForeignKey
from mptt.models import MPTTModel
from thx.core.models import CleanModel, DateModel, OrderedModel, StatusModel, UserModel

User = get_user_model()


class Role(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    @classmethod
    def get_roles(cls, user):
        return user.roles.all()

    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    app_label = models.CharField(
        max_length=100,
        verbose_name=_('app label')
    )
    model_name = models.CharField(
        max_length=100,
        verbose_name=_('model name')
    )
    codename = models.CharField(
        max_length=255,
        verbose_name=_('codename'),
    )
    users = models.ManyToManyField(
        User,
        related_name="roles",
        blank=True,
        verbose_name=_('users'),
    )

    class Meta:
        verbose_name = _('role')
        verbose_name_plural = _('roles')
        unique_together = (('app_label', 'model_name', 'codename'),)
        ordering = ('ordering', 'app_label', 'model_name', 'codename')
        permissions = (
            ("list_role", "Can list role"),
            ("detail_role", "Can detail role"),
            ("enable_role", "Can enable role"),
            ("disable_role", "Can disable role"),
        )

    def __str__(self):
        return "{} | {} | {}".format(
            self.app_label,
            self.model_name,
            self.name,
        )

    def clean_fields(self, exclude=None):
        self.set_codename()

    def clean(self):
        self.set_codename()
        self.set_app_label()
        self.set_model_name()
        super().clean()

    def set_codename(self):
        self.codename = slugify(self.name)

    def set_app_label(self):
        self.app_label = self.app_label.lower()

    def set_model_name(self):
        self.model_name = self.model_name.lower()


class UserRole(Role.users.through):
    class Meta:
        proxy = True
        ordering = ('role__app_label', 'role__model_name', 'role__name', 'role__name', 'user__username')

    @property
    def username(self):
        return self.user.username

    @property
    def email(self):
        return self.user.email

    @property
    def status(self):
        return self.role.status

    @property
    def app_label(self):
        return self.role.app_label

    @property
    def model_name(self):
        return self.role.model_name

    @property
    def name(self):
        return self.role.name


class RequiredField(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    role = models.ForeignKey(
        Role,
        on_delete=models.CASCADE,
        related_name='required_fields',
        verbose_name=_('role'),
    )
    from_status = models.CharField(
        max_length=50,
        verbose_name=_('from status'),
        help_text=_('e.g.: 1 -> "1", "create" -> "create"')
    )
    fields = JSONField(
        verbose_name=_('fields'),
        help_text=_('e.g.: ["field1", "field2"]')
    )

    class Meta:
        verbose_name = _('required field')
        verbose_name_plural = _('required fields')
        ordering = ('ordering', 'role__app_label', 'role__model_name', 'role__name', 'from_status')
        unique_together = (('role', 'from_status',),)
        permissions = (
            ("list_requiredfield", "Can list required field"),
            ("detail_requiredfield", "Can detail required field"),
            ("enable_requiredfield", "Can enable required field"),
            ("disable_requiredfield", "Can disable required field"),
        )

    def __str__(self):
        return "{} | {} | {} | {}".format(
            self.role.app_label,
            self.role.model_name,
            self.role.name,
            self.from_status
        )

    def clean(self):
        self.set_from_status()
        super().clean()

    def set_from_status(self):
        self.from_status = str(self.from_status)

    @property
    def app_label(self):
        return self.role.app_label

    @property
    def model_name(self):
        return self.role.model_name


class StatusFlow(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    role = models.ForeignKey(
        Role,
        on_delete=models.CASCADE,
        related_name='status_flows',
        verbose_name=_('role'),
    )
    from_status = models.CharField(
        max_length=50,
        verbose_name=_('from status'),
        help_text=_('e.g.: 1 -> "1", "create" -> "create"'),
    )
    to_status = JSONField(
        verbose_name=_('to status'),
        help_text=_('e.g.: ["status1", "status2"]'),
    )

    class Meta:
        verbose_name = _('status flow')
        verbose_name_plural = _('status flows')
        ordering = ('ordering', 'role__app_label', 'role__model_name', 'role__name', 'from_status')
        unique_together = (('role', 'from_status',),)
        permissions = (
            ("list_statusflow", "Can list status flow"),
            ("detail_statusflow", "Can detail status flow"),
            ("enable_statusflow", "Can enable status flow"),
            ("disable_statusflow", "Can disable status flow"),
        )

    def __str__(self):
        return "{} | {} | {} | {}".format(
            self.role.app_label,
            self.role.model_name,
            self.role.name,
            self.from_status
        )

    def clean(self):
        self.set_from_status()
        super().clean()

    def set_from_status(self):
        self.from_status = str(self.from_status)

    @property
    def app_label(self):
        return self.role.app_label

    @property
    def model_name(self):
        return self.role.model_name


class UsedField(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    role = models.ForeignKey(
        Role,
        on_delete=models.CASCADE,
        related_name='used_fields',
        verbose_name=_('role'),
    )
    from_status = models.CharField(
        max_length=50,
        verbose_name=_('from status'),
        help_text=_('e.g.: 1 -> "1", "create" -> "create"'),
    )
    action = models.CharField(
        max_length=50,
        verbose_name=_('action'),
        help_text=_('e.g.: "create", "enable", "destroy"'),
    )
    fields = JSONField(
        verbose_name=_('fields'),
        help_text=_('e.g.: ["field1", "field2"]')
    )

    class Meta:
        verbose_name = _('used field')
        verbose_name_plural = _('used fields')
        unique_together = (('role', 'from_status', 'action',),)
        ordering = (
            'ordering', 'role__app_label', 'role__model_name',
            'role__name', 'from_status', 'action'
        )
        permissions = (
            ("list_usedfield", "Can list used field"),
            ("detail_usedfield", "Can detail used field"),
            ("enable_usedfield", "Can enable used field"),
            ("disable_usedfield", "Can disable used field"),
        )

    def __str__(self):
        return "{} | {} | {} | {} | {}".format(
            self.role.app_label,
            self.role.model_name,
            self.role.name,
            self.from_status,
            self.action,
        )

    def clean(self):
        self.set_from_status()
        super().clean()

    def set_from_status(self):
        self.from_status = str(self.from_status)

    @property
    def app_label(self):
        return self.role.app_label

    @property
    def model_name(self):
        return self.role.model_name


class Filter(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    role = models.ForeignKey(
        Role,
        on_delete=models.CASCADE,
        related_name='filters',
        verbose_name=_('role'),
    )
    action = models.CharField(
        max_length=50,
        verbose_name=_('action'),
        help_text=_('e.g.: "create", "enable", "destroy"'),
    )
    codename = models.CharField(
        max_length=255,
        verbose_name=_('codename'),
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_('description'),
    )
    code = models.TextField(
        default="Q()",
        verbose_name=_('code'),
    )

    class Meta:
        verbose_name = _('filter')
        verbose_name_plural = _('filters')
        unique_together = (('role', 'codename'),)
        ordering = ('ordering', 'role__app_label', 'role__model_name', 'codename')
        permissions = (
            ("list_filter", "Can list filter"),
            ("detail_filter", "Can detail filter"),
            ("enable_filter", "Can enable filter"),
            ("disable_filter", "Can disable filter"),
        )

    def __str__(self):
        return "{} | {} | {}".format(
            self.role.app_label,
            self.role.model_name,
            self.action,
        )

    def clean_fields(self, exclude=None):
        self.set_codename()
        super().clean_fields(exclude)

    def set_codename(self):
        self.codename = slugify(self.action)

    @property
    def app_label(self):
        return self.role.app_label

    @property
    def model_name(self):
        return self.role.model_name


class Menu(MPTTModel, CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    parent = TreeForeignKey(
        "self",
        on_delete=models.CASCADE,
        related_name='children',
        null=True, blank=True,
        verbose_name=_('parent'),
    )
    title = models.CharField(
        max_length=255,
        verbose_name=_('title'),
    )
    language_code = models.CharField(
        max_length=2,
        verbose_name=_('language code'),
    )
    router = models.CharField(
        max_length=255,
        verbose_name=_('router'),
    )
    roles = models.ManyToManyField(
        Role,
        related_name='menus',
        blank=True,
        verbose_name=_('roles'),
    )
    info = JSONField(
        blank=True, null=True,
        verbose_name=_('info'),
    )

    class Meta:
        verbose_name = _('menu')
        verbose_name_plural = _('menu')
        ordering = ('ordering', 'language_code', 'parent__title', 'title')
        permissions = (
            ("list_menu", "Can list menu"),
            ("detail_menu", "Can detail menu"),
            ("enable_menu", "Can enable menu"),
            ("disable_menu", "Can disable menu"),
        )

    class MPTTMeta:
        parent_attr = 'parent'

    def __str__(self):
        return f"{self.parent} | {self.title}" if self.parent else f"{self.title}"

    def clean(self):
        self.set_language_code()
        super().clean()

    @classmethod
    def get_user_menu(cls, user, language_code, codename=None):
        language_code = language_code.lower()
        show_all_menus = getattr(settings, 'SHOW_ALL_MENUS', None)
        if show_all_menus:
            return Menu.objects.filter(language_code=language_code, status=1)
        items = Menu.objects.filter(roles__users=user, language_code=language_code, status=1)
        if codename:
            items = items.filter(roles__codename=codename)
        return items.distinct()

    def set_language_code(self):
        self.language_code = self.language_code.lower()

    def role_list(self):
        return [
            f"{obj.app_label} | {obj.model_name} | {obj.codename}"
            for obj in self.roles.filter(status=1)
        ]

    @property
    def parent_title(self):
        return self.parent.title if self.parent else None
